==> django2-auth0-auth-4.0.tar.gz: django2-auth0-auth-4.0/auth0_auth/backends.py <==
from base64 import urlsafe_b64encode
from hashlib import sha1
from uuid import uuid4

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .utils import (
    get_email_from_token,
    get_login_url,
    get_logout_url,
    is_email_verified_from_token,
    get_sub_from_token)

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User

    def get_user_model(*args, **kwargs):
        return User


if getattr(settings, "SESSION_COOKIE_SAMESITE"):
    raise ImproperlyConfigured(
        """Please set the SESSION_COOKIE_SAMESITE to None in your settings.py. To prevent replay 
        attacks the callback URL verifies a unique state string stored the session is equal value 
        that is posted back by Auth0."""
    )


class Auth0Backend(object):
    USER_CREATION = getattr(settings, "AUTH0_USER_CREATION", True)

    supports_anonymous_user = False
    supports_inactive_user = True
    supports_object_permissions = False

    def __init__(self):
        self.User = get_user_model()

    def login_url(self, redirect_uri, state):
        return get_login_url(redirect_uri=redirect_uri, state=state)

    def logout_url(self, redirect_uri):
        return get_logout_url(redirect_uri=redirect_uri)

    def authenticate(self, token=None, **kwargs):
        if token is None:
            return None

        email = get_email_from_token(token=token)
        sub = get_sub_from_token(token=token)

        if email is None:
            return None

        email_verified = is_email_verified_from_token(token=token)

        if not email_verified:
            return None

        user_model = get_user_model()

        user, created = user_model._default_manager.update_or_create(**{
            user_model.USERNAME_FIELD: sub,
            'defaults': {
                user_model.EMAIL_FIELD: email
            }
        })

        if not self.user_can_authenticate(user):
            return None
        user.backend = "{}.{}".format(
            self.__class__.__module__, self.__class__.__name__
        )
        return user

    def user_can_authenticate(self, user):
        is_active = getattr(user, "is_active", None)
        return is_active or is_active is None

    def get_user(self, user_id):
        try:
            user = self.User.objects.get(pk=user_id)
            return user
        except self.User.DoesNotExist:
            return None

    def create_user(self, email):
        if self.USER_CREATION:
            username_field = getattr(self.User, "USERNAME_FIELD", "username")
            user_kwargs = {"email": email}
            user_kwargs[username_field] = self.username_generator(email)
            return self.User.objects.create_user(**user_kwargs)
        else:
            return None

    @staticmethod
    def username_generator(email):
        return str(uuid4())

==> django2-auth0-auth-4.1.4.tar.gz: django2-auth0-auth-4.1.4/auth0_auth/backends.py <==
import datetime
from uuid import uuid4

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .utils import (
    get_email_from_token,
    get_login_url,
    get_logout_url,
    is_email_verified_from_token,
    get_sub_from_token, decode_jwt, get_jwt)

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User

    def get_user_model(*args, **kwargs):
        return User


if getattr(settings, "SESSION_COOKIE_SAMESITE"):
    raise ImproperlyConfigured(
        """Please set the SESSION_COOKIE_SAMESITE to None in your settings.py. To prevent replay 
        attacks the callback URL verifies a unique state string stored the session is equal value 
        that is posted back by Auth0."""
    )


class Auth0Backend(object):
    USER_CREATION = getattr(settings, "AUTH0_USER_CREATION", True)

    supports_anonymous_user = False
    supports_inactive_user = True
    supports_object_permissions = False

    def __init__(self, request=None):
        self.User = get_user_model()
        self.request = request

    def login_url(self, redirect_uri, state):
        return get_login_url(redirect_uri=redirect_uri, state=state)

    def logout_url(self, redirect_uri):
        return get_logout_url(redirect_uri=redirect_uri)

    def authenticate(self, code=None, redirect_uri=None, **kwargs):
        if code is None:
            return None

        if redirect_uri is None:
            return None

        resp = get_jwt(code, redirect_uri)
        self.request.session['auth'] = resp

        jwt = decode_jwt(resp['id_token'])
        self.request.session['jwt'] = jwt

        # Add a little buffer
        expires_at = datetime.datetime.now() + datetime.timedelta(seconds=resp['expires_in']*0.8)

        self.request.session['expires'] = expires_at.isoformat()

        email = get_email_from_token(jwt)
        sub = get_sub_from_token(jwt)

        if email is None:
            return None

        email_verified = is_email_verified_from_token(jwt)

        if not email_verified:
            return None

        user_model = get_user_model()

        user, created = user_model._default_manager.update_or_create(**{
            user_model.USERNAME_FIELD: sub,
            'defaults': {
                user_model.EMAIL_FIELD: email
            }
        })

        if not self.user_can_authenticate(user):
            return None
        user.backend = "{}.{}".format(
            self.__class__.__module__, self.__class__.__name__
        )
        return user

    def user_can_authenticate(self, user):
        is_active = getattr(user, "is_active", None)
        return is_active or is_active is None

    def get_user(self, user_id):
        try:
            user = self.User.objects.get(pk=user_id)
            return user
        except self.User.DoesNotExist:
            return None

    def create_user(self, email):
        if self.USER_CREATION:
            username_field = getattr(self.User, "USERNAME_FIELD", "username")
            user_kwargs = {"email": email}
            user_kwargs[username_field] = self.username_generator(email)
            return self.User.objects.create_user(**user_kwargs)
        else:
            return None

    @staticmethod
    def username_generator(email):
        return str(uuid4())
